import define from "@/server/api/define.js";
import { ApiError } from "@/server/api/error.js";
import { Clips } from "@/models/index.js";

export const meta = {
	tags: ["clips"],

	requireCredential: true,

	kind: "write:account",

	errors: {
		noSuchClip: {
			message: "No such clip.",
			code: "NO_SUCH_CLIP",
			id: "70ca08ba-6865-4630-b6fb-8494759aa754",
		},
	},
} as const;

export const paramDef = {
	type: "object",
	properties: {
		clipId: { type: "string", format: "misskey:id" },
	},
	required: ["clipId"],
} as const;

export default define(meta, paramDef, async (ps, user) => {
	const clip = await Clips.findOneBy({
		id: ps.clipId,
		userId: user.id,
	});

	if (clip == null) {
		throw new ApiError(meta.errors.noSuchClip);
	}

	await Clips.delete(clip.id);
});
