import define from "@/server/api/define.js";
import { DriveFiles } from "@/models/index.js";
import { makePaginationQuery } from "@/server/api/common/make-pagination-query.js";

export const meta = {
	tags: ["drive"],

	requireCredential: true,

	kind: "read:drive",

	res: {
		type: "array",
		optional: false,
		nullable: false,
		items: {
			type: "object",
			optional: false,
			nullable: false,
			ref: "DriveFile",
		},
	},
} as const;

export const paramDef = {
	type: "object",
	properties: {
		limit: { type: "integer", minimum: 1, maximum: 100, default: 10 },
		sinceId: { type: "string", format: "misskey:id" },
		untilId: { type: "string", format: "misskey:id" },
		type: {
			type: "string",
			pattern: /^[a-zA-Z\/\-*]+$/.toString().slice(1, -1),
		},
	},
	required: [],
} as const;

export default define(meta, paramDef, async (ps, user) => {
	const query = makePaginationQuery(
		DriveFiles.createQueryBuilder("file"),
		ps.sinceId,
		ps.untilId,
	).andWhere("file.userId = :userId", { userId: user.id });

	if (ps.type) {
		if (ps.type.endsWith("/*")) {
			query.andWhere("file.type like :type", {
				type: `${ps.type.replace("/*", "/")}%`,
			});
		} else {
			query.andWhere("file.type = :type", { type: ps.type });
		}
	}

	const files = await query.take(ps.limit).getMany();

	return await DriveFiles.packMany(files, { detail: false, self: true });
});
